/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (C) 2006 Mark Doliner
 * Copyright (C) 2006 John Lewis
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Ant" and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package net.sourceforge.cobertura.ant;

import org.apache.tools.ant.taskdefs.Java;
import org.apache.tools.ant.types.Environment.Variable;

abstract class AntUtil {

	private static final String COBERTURA_DATAFILE = "net.sourceforge.cobertura.datafile";

	/**
	 * Used to transfer the net.sourceforge.cobertura.datafile property to a JVM
	 * that is about to be forked.
	 * <p/>
	 * This is confusing, but it's required by our functional test.
	 * What happens is, we have a JUnit test that runs ant to
	 * instrument some classes.  When the instrumentation is running,
	 * we want to get the coverage info that is created by exercising
	 * our instrumentation classes.
	 * <p/>
	 * So we pass in two different coverage files:
	 * 1. The coverage data file command line parameter.  This tells
	 * the instrument task where to write the new coverage data.
	 * 2. The coverage data system property.  This tells the
	 * instrumentation inside the instrumented classes where to
	 * keep track of the line hit counts, etc.
	 *
	 * @param task The Java task that will do the forking.
	 */
	static void transferCoberturaDataFileProperty(Java task) {
		String coberturaProperty = System.getProperty(COBERTURA_DATAFILE);
		if (coberturaProperty != null) {
			Variable sysproperty = new Variable();
			sysproperty.setKey(COBERTURA_DATAFILE);
			sysproperty.setValue(coberturaProperty);
			task.addSysproperty(sysproperty);
		}
	}

}
